@@ -1,13 +1,12 @@
 /*
- * (getSC) Copyright 1996-2016 ECMWF.
+ * (C) Copyright 1996-2016 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
  * In applying this licence, ECMWF does not waive the privileges and immunities
- * granted to it by virtue of its status as an intergovernmental organisation nor
- * does it submit to any jurisdiction.
+ * granted to it by virtue of its status as an intergovernmental organisation
+ * nor does it submit to any jurisdiction.
  */
-
 /*! \file GribDecoder.cc
     \brief Implementation of the class GribDecoder.
 
@@ -20,9 +19,10 @@
  */
 
 #include "GribDecoder.h"
-#include <limits>
-#include "Factory.h"
 
+#include <limits>
+
+#include "Factory.h"
 #include "magics_windef.h"
 #ifndef MAGICS_ON_WINDOWS
 #include <unistd.h>
@@ -31,10 +31,10 @@
 #endif
 
 #include <cerrno>
+
 #include "AnimationRules.h"
 #include "GribInterpretor.h"
 #include "MagDateTime.h"
-
 #include "MagJSon.h"
 #include "MetaData.h"
 #include "TextVisitor.h"
@@ -139,8 +139,13 @@ long computeStep(const GribDecoder& grib, const string& key) {
         stepUnits["30Y"] = stepUnits["M"] * 30;
         stepUnits["C"]   = stepUnits["M"] * 100;
     }
-    string units = grib.getstring("stepUnits");
-    long step    = grib.getLong(key);
+    string units;
+    if (grib.getstring("indicatorOfUnitOfTimeRange") == "m")
+        units = "m";
+    else
+        units = grib.getstring("stepUnits");
+
+    long step = grib.getLong(key);
 
     map<string, double>::iterator stepunit = stepUnits.find(units);
     double factor                          = 1;
@@ -161,8 +166,7 @@ long GribDecoder::getLong(const string& key, bool warnIfKeyAbsent) const {
     int err = grib_get_long(current_handle_, key.c_str(), &val);
     if (err) {
         if (warnIfKeyAbsent) {
-            MagLog::warning() << "Grib API: can not find key [" << key << "]  - " << grib_get_error_message(err)
-                              << "\n";
+            MagLog::warning() << "ecCodes: cannot find key [" << key << "]  - " << grib_get_error_message(err) << "\n";
         }
         return 0;
     }
@@ -182,13 +186,11 @@ string GribDecoder::getstring(const string& key, bool warnIfKeyAbsent, bool cach
     char val[1024];
     size_t length = 1024;
 
-
     int err = grib_get_string(current_handle_, key.c_str(), val, &length);
 
     if (err) {
         if (warnIfKeyAbsent) {
-            MagLog::warning() << "Grib API: can not find key [" << key << "]  - " << grib_get_error_message(err)
-                              << "\n";
+            MagLog::warning() << "ecCodes: cannot find key [" << key << "]  - " << grib_get_error_message(err) << "\n";
         }
         return "";
     }
@@ -225,7 +227,6 @@ string GribDecoder::getString(const string& key, bool warnIfKeyAbsent) const {
     return value;
 }
 
-
 double GribDecoder::getDouble(const string& key, bool warnIfKeyAbsent) const {
     if (!valid_)
         return 0;
@@ -237,8 +238,7 @@ double GribDecoder::getDouble(const string& key, bool warnIfKeyAbsent) const {
     int err = grib_get_double(current_handle_, key.c_str(), &val);
     if (err) {
         if (warnIfKeyAbsent) {
-            MagLog::warning() << "Grib API: can not find key [" << key << "]  - " << grib_get_error_message(err)
-                              << "\n";
+            MagLog::warning() << "ecCodes: cannot find key [" << key << "]  - " << grib_get_error_message(err) << "\n";
         }
         return 0;
     }
@@ -246,17 +246,16 @@ double GribDecoder::getDouble(const string& key, bool warnIfKeyAbsent) const {
     return val;
 }
 
-
 void GribDecoder::setDouble(const string& key, double val) const {
     int err = grib_set_double(current_handle_, key.c_str(), val);
     if (err) {
-        MagLog::warning() << "Grib API: can not find key [" << key << "]  - " << grib_get_error_message(err) << "\n";
+        MagLog::warning() << "ecCodes: cannot find key [" << key << "]  - " << grib_get_error_message(err) << "\n";
     }
 
     if (component2_)
         err = grib_set_double(component2_, key.c_str(), val);
     if (err) {
-        MagLog::warning() << "Grib API: can not find key [" << key << "]  - " << grib_get_error_message(err) << "\n";
+        MagLog::warning() << "ecCodes: cannot find key [" << key << "]  - " << grib_get_error_message(err) << "\n";
     }
 }
 
@@ -264,7 +263,6 @@ void GribDecoder::scale(const string& metadata, double& scaling, double& offset)
     assert(false);
 }
 
-
 bool GribDecoder::isEarthOblate() const {
     // if getLong does not find the key, then it returns zero
     // hence, we return true only if the key exists and is 1
@@ -275,7 +273,6 @@ bool GribDecoder::isEarthOblate() const {
     return false;
 }
 
-
 Matrix* GribDecoder::colour(Matrix* matrix) {
     if (cHandle()) {
         colourComponent_ = matrix;
@@ -312,19 +309,20 @@ void GribDecoder::read() {
         return;
     }
 
-    current_handle_             = field_;
-    const string representation = getstring("typeOfGrid");  // Make sure we only get one info
+    current_handle_   = field_;
+    const string grid = representation();
+
 
     try {
         if (!interpretor_) {
-            interpretor_ = SimpleObjectMaker<GribInterpretor>::create(representation);
+            interpretor_ = SimpleObjectMaker<GribInterpretor>::create(grid);
         }
         interpretor_->interpretAsMatrix(*this);
         Matrix* matrix = u();
         if (u() == 0) {
             valid_ = false;
             ostringstream msg;
-            msg << "Grib Decoder: Representation [" << representation << "] not yet fully implemented";
+            msg << "Grib Decoder: Representation [" << grid << "] not yet fully implemented";
             MagLog::error() << msg.str() << std::endl;
             throw MagicsException(msg.str());
         }
@@ -332,38 +330,36 @@ void GribDecoder::read() {
     }
     catch (NoFactoryException&) {
         ostringstream msg;
-        msg << "Grib Decoder - read: Representation [" << representation << "] not supported";
+        msg << "Grib Decoder - read: Representation [" << grid << "] not supported";
         MagLog::error() << msg.str() << endl;
         valid_ = false;
         throw MagicsException(msg.str());
     }
 }
 
-
 void GribDecoder::read(const Transformation& transformation) {
     if (!field_) {
         return;
     }
-    const string representation = getstring("typeOfGrid");
+    const string grid = representation();
+
 
     try {
         if (!interpretor_) {
-            interpretor_ = SimpleObjectMaker<GribInterpretor>::create(representation);
+            interpretor_ = SimpleObjectMaker<GribInterpretor>::create(grid);
         }
         interpretor_->interpretAsMatrix(*this, transformation);
         Matrix* matrix = u();
         interpretor_->scaling(*this, *matrix);
     }
     catch (NoFactoryException&) {
-        MagLog::error() << "Grib Decoder - read: Representation [" << representation << "] not supported.\n"
-                        << std::endl;
+        MagLog::error() << "Grib Decoder - read: Representation [" << grid << "] not supported.\n" << std::endl;
         ;
         valid_ = false;
         throw MagicsException("Grib Decoder: Representation [] not supported.");
     }
 }
 
-
 /*!
  Class information are given to the output-stream.
  */
@@ -428,38 +424,35 @@ void GribDecoder::decode2D() {
     openThirdComponent();
     read();
 
-    const string representation = getstring("typeOfGrid");
+    const string grid = representation();
     try {
         if (!interpretor_) {
-            interpretor_ = SimpleObjectMaker<GribInterpretor>::create(representation);
+            interpretor_ = SimpleObjectMaker<GribInterpretor>::create(grid);
         }
         interpretor_->keepOriginal(true);
     }
     catch (NoFactoryException&) {
-        MagLog::warning() << "Grib Decoder: Vector cobination of representations [" << representation
-                          << "] not supported.\n"
+        MagLog::warning() << "Grib Decoder: Vector cobination of representations [" << grid << "] not supported.\n"
                           << std::endl;
         ;
         valid_ = false;
     }
 
-
     Data::dimension_ = (colourComponent_) ? 3 : 2;
 
     wind_mode_->x(*xComponent_, *yComponent_);
     interpretor_->keepOriginal(false);
 }
 
-
 void GribDecoder::customisedPoints(const AutomaticThinningMethod& thinning, const Transformation& transformation,
                                    const std::set<string>& request, CustomisedPointsList& points) {
     decode();
     long repres;
     grib_get_long(field_, "dataRepresentationType", &repres);
-    const string representation = getstring("typeOfGrid");
+    const string grid = representation();
     try {
         if (!interpretor_) {
-            interpretor_ = SimpleObjectMaker<GribInterpretor>::create(representation);
+            interpretor_ = SimpleObjectMaker<GribInterpretor>::create(grid);
         }
 
         // Compute the thinning factor...
@@ -485,14 +478,13 @@ void GribDecoder::customisedPoints(const AutomaticThinningMethod& thinning, cons
         customisedPoints(transformation, points, xstep, ystep, 0);
     }
     catch (NoFactoryException&) {
-        MagLog::error() << "Grib Decoder - customisedPoints: Representation [" << representation << "] not supported.\n"
+        MagLog::error() << "Grib Decoder - customisedPoints: Representation [" << grid << "] not supported.\n"
                         << std::endl;
         ;
         throw MagicsException("Grib Decoder: Representation [] not supported.");
     }
 }
 
-
 grib_handle* GribDecoder::uHandle(string& name) {
     name = "x_component";
     return field_;
@@ -508,14 +500,12 @@ grib_handle* GribDecoder::cHandle(string& name) {
     return colour_;
 }
 
-
 bool compare(const pair<double, double>& pt1, const pair<double, double>& pt2) {
     if (pt1.second != pt2.second)
         return false;
     return pt1.second < pt2.second;
 }
 
-
 void GribDecoder::newPoint(const Transformation& transformation, double lat, double lon, double uc, double vc,
                            double cc, vector<CustomisedPoint*>& points, double grid) {
     std::stack<UserPoint> duplicates;
@@ -562,7 +552,6 @@ void GribDecoder::newPoint(const Transformation& transformation, double lat, dou
     }
 }
 
-
 struct Compare {
     template <typename T1, typename T2>
     bool operator()(T1 const& t1, T2 const& t2) const {
@@ -574,13 +563,11 @@ void GribDecoder::customisedPoints(const Transformation& transformation, Customi
                                    double thiny, double gap) {
     decode2D();
 
-
     double minlon = 0.;
     double maxlon = 360.;
 
     double missing = getDouble("missingValue");
 
-
     if (thiny && thinning_) {
         vector<pair<double, double> > positions;
 
@@ -674,16 +661,15 @@ void GribDecoder::customisedPoints(const Transformation& transformation, Customi
     }
 }
 
-
 void GribDecoder::customisedPoints(const BasicThinningMethod& thinning, const Transformation& transformation,
                                    const std::set<string>& needs, CustomisedPointsList& points) {
     decode();
     long repres;
     grib_get_long(field_, "dataRepresentationType", &repres);
-    const string representation = getstring("typeOfGrid");
+    const string grid = representation();
     try {
         if (!interpretor_) {
-            interpretor_ = SimpleObjectMaker<GribInterpretor>::create(representation);
+            interpretor_ = SimpleObjectMaker<GribInterpretor>::create(grid);
         }
 
         // Compute the thinning factor...
@@ -702,14 +688,13 @@ void GribDecoder::customisedPoints(const BasicThinningMethod& thinning, const Tr
         customisedPoints(transformation, points, gap * thinning.factor(), gap * thinning.factor(), gap);
     }
     catch (NoFactoryException&) {
-        MagLog::error() << "Grib Decoder- customisedPoints: Representation [" << representation << "] not supported.\n"
+        MagLog::error() << "Grib Decoder- customisedPoints: Representation [" << grid << "] not supported.\n"
                         << std::endl;
         ;
         throw MagicsException("Grib Decoder: Representation [] not supported.");
     }
 }
 
-
 void GribDecoder::decode2D(const Transformation&) {
     Data::dimension_ = 2;
     if (xComponent_)
@@ -747,7 +732,6 @@ grib_handle* GribEntryDecoder::open(grib_handle* handle, bool) {
     return handle;
 }
 
-
 grib_handle* GribDecoder::open(grib_handle* grib, bool sendmsg) {
     if (grib) {
         return grib;
@@ -759,7 +743,6 @@ grib_handle* GribDecoder::open(grib_handle* grib, bool sendmsg) {
     _set_fmode(_O_BINARY);
 #endif
 
-
     FILE* file = fopen(file_name_.c_str(), "r");
 
     if (!file) {
@@ -771,7 +754,6 @@ grib_handle* GribDecoder::open(grib_handle* grib, bool sendmsg) {
         return 0;
     }
 
-
     if (loop_) {
         grib_context* context = grib_context_get_default();
         int error             = 0;
@@ -791,7 +773,6 @@ grib_handle* GribDecoder::open(grib_handle* grib, bool sendmsg) {
         return first;
     }
 
-
     grib_handle* handle = (*address_mode_)(0, file, current_position_);
 
     if (!handle) {
@@ -816,7 +797,6 @@ grib_handle* GribDecoder::open(grib_handle* grib, bool sendmsg) {
     return handle;
 }
 
-
 grib_nearest* GribDecoder::nearest_point_handle(bool keep) {
     int err;
 
@@ -825,7 +805,8 @@ grib_nearest* GribDecoder::nearest_point_handle(bool keep) {
     }
     else {  // we want to retain this and only create a new one if needed
         if (!nearest_) {
-            nearest_ = grib_nearest_new(current_handle_, &err);  // only used in Metview's Cursor Data facility
+            nearest_ = grib_nearest_new(current_handle_,
+                                        &err);  // only used in Metview's Cursor Data facility
             if (err)
                 return 0;
         }
@@ -833,7 +814,6 @@ grib_nearest* GribDecoder::nearest_point_handle(bool keep) {
     }
 }
 
-
 bool GribDecoder::id(const string& id, const string& where) const {
     if (id_.empty() && id.empty()) {
         return (verify(where));
@@ -841,9 +821,9 @@ bool GribDecoder::id(const string& id, const string& where) const {
     return magCompare(id_, id);
 }
 
-
 bool GribDecoder::verify(const string& val) const {
-    // we except a string with the following format "key1=val,key2=val2,...,keyn=valn"
+    // we except a string with the following format
+    // "key1=val,key2=val2,...,keyn=valn"
     Tokenizer tokenizer(",= ");
     vector<string> tokens;
     map<string, string> where;
@@ -881,17 +861,16 @@ void GribDecoder::decodePoints() {
     if (Data::dimension_ == 1) {
         double scaling;
         double offset;
-        const string representation = getstring("typeOfGrid");
-        double missing              = getDouble("missingValue");
+        const string grid = representation();
+        double missing    = getDouble("missingValue");
         try {
             if (!interpretor_) {
-                interpretor_ = SimpleObjectMaker<GribInterpretor>::create(representation);
+                interpretor_ = SimpleObjectMaker<GribInterpretor>::create(grid);
             }
             interpretor_->scaling(*this, scaling, offset);
         }
         catch (NoFactoryException&) {
-            MagLog::warning() << "Grib Decoder: Representation [" << representation << "] not supported.\n"
-                              << std::endl;
+            MagLog::warning() << "Grib Decoder: Representation [" << grid << "] not supported.\n" << std::endl;
             ;
             scaling = 1;
             offset  = 0;
@@ -916,8 +895,7 @@ void GribDecoder::decodePoints() {
     }
 
 
-    const string representation = getstring("typeOfGrid");
-    double missing              = getDouble("missingValue");
+    double missing = getDouble("missingValue");
 
     grib_iterator* iter1 = grib_iterator_new(field_, flags, &error);
     grib_iterator* iter2 = grib_iterator_new(component2_, flags, &error);
@@ -973,17 +951,14 @@ GribLoop::~GribLoop() {
     }
 }
 
-
 Data* GribLoop::current() {
     return currentgrib_;
 }
 
-
 map<string, string> GribLoop::ids_;
 int GribLoop::index_ = 0;
 void GribLoop::next() {}
 
-
 GribLoop::GribLoop() : currentgrib_(0), file_(0) {
     currentDim_ = dimension_.begin();
     currentPos_ = dim_.begin();
@@ -992,13 +967,11 @@ GribLoop::GribLoop() : currentgrib_(0), file_(0) {
     counter_  = 0;
 }
 
-
 void GribLoop::setToFirst() {
     currentDim_ = dimension_.begin();
     currentPos_ = dim_.begin();
 }
 
-
 bool GribLoop::hasMore() {
     if (file_ == 0) {
         file_ = fopen(path_.c_str(), "r");
@@ -1039,7 +1012,6 @@ bool GribLoop::hasMore() {
         }
     }
 
-
     if (*currentDim_ == 2) {
         if (dim_.empty()) {
             // case 2 Dimension = 2 and loop on all the field!
@@ -1065,7 +1037,6 @@ bool GribLoop::hasMore() {
                 return false;
             currentPos_++;
 
-
             grib_handle* handle1 = (*address_mode_)(0, file_, *dim1);
             grib_handle* handle2 = (*address_mode_)(0, file_, *dim2);
             if (!handle1)
@@ -1202,7 +1173,6 @@ public:
         const long mn   = grib_.getLong("minute");
         const long step = computeStep(grib_, "stepRange");  // default is in hours. Set 'stepUnits' to change.
 
-
         MagDate part1 = MagDate(day);
         MagTime part2 = MagTime(hour, mn, 0);
         DateTime full(part1, part2);
@@ -1355,13 +1325,12 @@ protected:
 
 void GribDecoder::visit(AnimationRules&) {}
 
-
 // GribDecoder::nearestGridpoints
-// For a list of input gridpoint locations, returns the locations and values for the closest actual data
-// point to each.
-// Note that we cannot yet apply the same method to all grid types because there is a bug in GRIB_API versions
-// prior to 1.15.0 which means that grib_nearest_find() does not work with Lambert grids when we keep the
-// same grib_nearest object.
+// For a list of input gridpoint locations, returns the locations and values for
+// the closest actual data point to each. Note that we cannot yet apply the same
+// method to all grid types because there is a bug in GRIB_API versions prior
+// to 1.15.0 which means that grib_nearest_find() does not work with Lambert
+// grids when we keep the same grib_nearest object.
 void GribDecoder::nearestGridpoints(double* inlats, double* inlons, double* outlats, double* outlons, double* values,
                                     double* distances, int nb, const string& representation, double missing) {
     bool retainGribNearestHandle = false;
@@ -1392,9 +1361,18 @@ void GribDecoder::nearestGridpoints(double* inlats, double* inlons, double* outl
 
     for (int i = 0; i < nb; i++) {
         if (nearHandle) {
-            // TODO: Should really catch the error (return value) from grib_nearest_find
-            grib_nearest_find(nearHandle, field_, inlats[i], inlons[i], GRIB_NEAREST_SAME_GRID, outlats4, outlons4,
-                              outvals4, outdist4, outindexes, &len);
+            // TODO: Should really catch the error (return value) from
+            // grib_nearest_find
+
+            int err = codes_grib_nearest_find(nearHandle, field_, inlats[i], inlons[i], GRIB_NEAREST_SAME_GRID,
+                                              outlats4, outlons4, outvals4, outdist4, outindexes, &len);
+            if (err) {
+                outlats[i]   = missing;
+                outlons[i]   = missing;
+                values[i]    = missing;
+                distances[i] = missing;
+            }
+
             vector<double> vdistances(outdist4, outdist4 + 4);
             int closestIndex = distance(vdistances.begin(), min_element(vdistances.begin(), vdistances.end()));
             outlats[i]       = outlats4[closestIndex];
@@ -1410,12 +1388,10 @@ void GribDecoder::nearestGridpoints(double* inlats, double* inlons, double* outl
         }
     }
 
-
     if (!retainGribNearestHandle && nearHandle)  // was this a temporary handle?
         grib_nearest_delete(nearHandle);
 }
 
-
 void GribDecoder::visit(ValuesCollector& points) {
     decode();
 
@@ -1433,18 +1409,18 @@ void GribDecoder::visit(ValuesCollector& points) {
 
     double scaling, offset;
     string oriUnits, derivedUnits;
-    string representation = getstring("typeOfGrid");
+    string grid = representation();
 
     // Scaling works only for scalar data!!!
 
     try {
         if (!interpretor_) {
-            interpretor_ = SimpleObjectMaker<GribInterpretor>::create(representation);
+            interpretor_ = SimpleObjectMaker<GribInterpretor>::create(grid);
         }
         interpretor_->scaling(*this, scaling, offset, oriUnits, derivedUnits);
     }
     catch (NoFactoryException&) {
-        MagLog::warning() << "Grib Decoder: Representation [" << representation << "] not supported.\n" << std::endl;
+        MagLog::warning() << "Grib Decoder: Representation [" << grid << "] not supported.\n" << std::endl;
         ;
         scaling = 1;
         offset  = 0;
@@ -1466,8 +1442,7 @@ void GribDecoder::visit(ValuesCollector& points) {
         points.setUnits(oriUnits);
         points.setScaledUnits(derivedUnits);
 
-
-        nearestGridpoints(inlats, inlons, outlats, outlons, values, distances, nb, representation, missing);
+        nearestGridpoints(inlats, inlons, outlats, outlons, values, distances, nb, grid, missing);
 
         for (int i = 0; i < nb; i++) {
             points[i].push_back(new ValuesCollectorData(outlons[i], outlats[i], values[i], distances[i]));
@@ -1485,9 +1460,8 @@ void GribDecoder::visit(ValuesCollector& points) {
         points.setUnits(oriUnits);
         points.setScaledUnits("/");
 
-
-        nearestGridpoints(inlats, inlons, outlats, outlons, x, distances, nb, representation, missing);
-        nearestGridpoints(inlats, inlons, outlats, outlons, y, distances, nb, representation, missing);
+        nearestGridpoints(inlats, inlons, outlats, outlons, x, distances, nb, grid, missing);
+        nearestGridpoints(inlats, inlons, outlats, outlons, y, distances, nb, grid, missing);
 
         for (int i = 0; i < nb; i++) {
             points[i].push_back(wind_mode_->values(outlons[i], outlats[i], x[i], y[i], distances[i]));
@@ -1572,6 +1546,14 @@ void GribDecoder::visit(MetaDataVisitor& meta) {
     meta.add("grib", grib.str());
 }
 
+string GribDecoder::representation() {
+    string grid = getstring("typeOfGrid");
+    string proj = getstring("projTargetString");
+
+
+    return (proj.size()) ? "proj" : grid;
+}
+
 void GribDecoder::visit(MetaDataCollector& step) {
     // Here we gather information for the label!
     const Transformation& transformation = step.transformation();
@@ -1604,7 +1586,6 @@ void GribDecoder::visit(MetaDataCollector& step) {
                 if (step.attribute(key->first).group() == MetaDataAttribute::StatsGroup) {
                     stats_.clear();
 
-
                     PointsHandler& points = this->points(transformation, false);
 
                     points.setToFirst();
@@ -1629,10 +1610,9 @@ void GribDecoder::visit(MetaDataCollector& step) {
                 else if (key->first == "scaling_formula" || key->first == "scaled_units") {
                     double scaling, offset;
                     string oriUnits, derivedUnits;
-                    string representation = getstring("typeOfGrid");
+                    string grid = representation();
                     try {
-                        unique_ptr<GribInterpretor> interpretor_(
-                            SimpleObjectMaker<GribInterpretor>::create(representation));
+                        unique_ptr<GribInterpretor> interpretor_(SimpleObjectMaker<GribInterpretor>::create(grid));
                         interpretor_->scaling(*this, scaling, offset, oriUnits, derivedUnits);
                         if (scaling == 1 && offset == 0) {
                             information_["scaling_formula"] = "";
@@ -1646,7 +1626,7 @@ void GribDecoder::visit(MetaDataCollector& step) {
                         }
                     }
                     catch (NoFactoryException&) {
-                        MagLog::warning() << "Grib Decoder: Representation [" << representation << "] not supported.\n"
+                        MagLog::warning() << "Grib Decoder: Representation [" << grid << "] not supported.\n"
                                           << std::endl;
                         ;
                         information_[key->first] = "N/A";
@@ -1657,8 +1637,8 @@ void GribDecoder::visit(MetaDataCollector& step) {
                     // Octahedral reduced Gaussian grids:
                     // - only reduced Gaussian grids can be octahedral
                     // - the isOctahedral key only works properly with global fields
-                    string representation = getstring("typeOfGrid");
-                    if (representation == "reduced_gg") {
+                    string grid = getstring("typeOfGrid");
+                    if (grid == "reduced_gg") {
                         if (getLong("global") == 1) {
                             information_["isOctahedral"] = (getLong("isOctahedral") == 1) ? "yes" : "no";
                         }
@@ -1672,7 +1652,6 @@ void GribDecoder::visit(MetaDataCollector& step) {
             }
         }
 
-
         if (!need.empty()) {
             TagHandler helper;
             GribTag tag1(*this, helper);
@@ -1707,7 +1686,6 @@ MatrixHandler& GribDecoder::direction() {
     // Now X et X components are ready ..
     // We compute the direction in xComponent_  and send it back.
 
-
     if (directionDone_)
         return *(matrixHandlers_.back());
     directionDone_                   = true;
@@ -1724,7 +1702,6 @@ MatrixHandler& GribDecoder::direction() {
     }
     xComponent_->clear();
 
-
     for (vector<double>::iterator d = directions.begin(); d != directions.end(); ++d) {
         xComponent_->push_back(*d);
     }
@@ -1737,7 +1714,6 @@ void GribDecoder::decode(const Transformation& transformation) {
     if (xComponent_ || !valid_)
         return;
 
-
     read(transformation);
 
     if (!xComponent_)
@@ -1801,12 +1777,10 @@ void GribDecoder::decode() {
     to_      = DateTime(helper.get("grib" + id_, "end-date"));
 }
 
-
 void GribDecoder::visit(TextVisitor& title) {
     if (!valid_)
         return;
 
-
     vector<string> titles;
 
     title.titles(titles);
@@ -1820,22 +1794,21 @@ void GribDecoder::visit(TextVisitor& title) {
 void GribDecoder::decodeRaster(const Transformation& transformation) {
     decode();
 
-    string representation = getstring("typeOfGrid");
+    string grid = representation();
 
     try {
         if (!interpretor_) {
-            interpretor_ = SimpleObjectMaker<GribInterpretor>::create(representation);
+            interpretor_ = SimpleObjectMaker<GribInterpretor>::create(grid);
         }
         interpretor_->interpretAsRaster(*this, raster_, transformation);
     }
 
     catch (NoFactoryException&) {
-        MagLog::error() << "Grib Decoder: Raster representation [" << representation << "] not supported.\n";
+        MagLog::error() << "Grib Decoder: Raster representation [" << grid << "] not supported.\n";
         throw MagicsException("Grib Decoder: Raster representation [] not supported.");
     }
 }
 
-
 void GribDecoder::initInfo() {
 #if defined(HAVE_METVIEW) && !defined(MAGICS_ON_WINDOWS)
     if (information_.find("_datatype") == information_.end()) {
@@ -1855,10 +1828,8 @@ void GribDecoder::initInfo() {
 #endif
 }
 
-
 namespace magics {
 
-
 class GribInfo {
 public:
     GribInfo() {}
@@ -1904,12 +1875,12 @@ public:
     GribObsDiagCriter() {}
     ~GribObsDiagCriter() {}
     bool verify(const GribDecoder& grib, const string&, const string&) {
-        string param = grib.getstring("observationDiagnostic", false);  // do not warn if the key is absent
+        string param = grib.getstring("observationDiagnostic",
+                                      false);  // do not warn if the key is absent
         return (param != "");
     }
 };
 
-
 class GribLocalDefHandler : public TitleFieldHandler {
 public:
     GribLocalDefHandler() {}
@@ -1978,7 +1949,6 @@ public:
     ~GribTypeHandler() {}
 };
 
-
 class GribParamHandler : public TitleFieldHandler {
 public:
     GribParamHandler() {}
@@ -1989,7 +1959,6 @@ public:
     }
 };
 
-
 class GribKeyHandler : public TitleFieldHandler {
 public:
     GribKeyHandler() {}
@@ -2006,7 +1975,6 @@ public:
     }
 };
 
-
 class GribBaseDateHandler : public TitleFieldHandler {
 public:
     GribBaseDateHandler() {}
@@ -2019,14 +1987,13 @@ public:
         long mn   = grib.getLong("minute");
         string x  = grib.getstring("dataDate");
 
-
         MagDate part1 = MagDate(date);
         MagTime part2 = MagTime(hour, mn, 0);
         DateTime full(part1, part2);
         const long type = grib.getLong("significanceOfReferenceTime", false);
-        if (type == 2) {  //     Verifying time of forecast
-            long step =
-                grib.getLong("step");  // needs steps in second!   // default is in hours. Set 'stepUnits' to change.
+        if (type == 2) {                       //     Verifying time of forecast
+            long step = grib.getLong("step");  // needs steps in second!   // default is in
+                                               // hours. Set 'stepUnits' to change.
             full = full + (step * -3600);
         }
 
@@ -2036,7 +2003,6 @@ public:
     }
 };
 
-
 class GribValidDateHandler : public TitleFieldHandler {
 public:
     GribValidDateHandler() {}
@@ -2047,7 +2013,6 @@ public:
         long hour = grib.getLong("hour");
         long mn   = grib.getLong("minute");
 
-
         long step = computeStep(grib, "step");
 
         MagDate part1 = MagDate(date);
@@ -2058,7 +2023,6 @@ public:
             full = full + step;
         }
 
-
         string format = field.attribute("format", "%A %d %B %Y %H%M UTC");
 
         title.back() += full.tostring(format);
@@ -2074,7 +2038,6 @@ public:
         long startstep = grib.getLong("startStep");
         long endstep   = grib.getLong("endStep");
 
-
         if (startstep != endstep) {
             ostringstream step;
             step << "from t+" << startstep << " to t+" << endstep;
@@ -2082,7 +2045,6 @@ public:
             return;
         }
 
-
         ostringstream step;
         step << startstep;
         string format = field.attribute("format", "t+%s");
@@ -2091,7 +2053,6 @@ public:
     }
 };
 
-
 class GribLevelHandler : public TitleFieldHandler {
 public:
     GribLevelHandler() {
@@ -2113,7 +2074,6 @@ public:
 
         string level = grib.getstring("typeOfLevel");
 
-
         map<string, GribLevelHandler::Builder>::iterator help = map_.find(level);
         if (help != map_.end())
             out << (this->*help->second)(level, grib) << " ";
@@ -2160,7 +2120,6 @@ protected:
 };
 map<string, GribLevelHandler::Builder> GribLevelHandler::map_;
 
-
 class GribTimeHandler : public TitleFieldHandler {
 public:
     GribTimeHandler() {}
@@ -2193,7 +2152,6 @@ public:
     }
 };
 
-
 class GribPlotTypeHandler : public TitleFieldHandler {
 public:
     GribPlotTypeHandler() {}
@@ -2201,7 +2159,6 @@ public:
     void operator()(TitleField&, vector<string>&, const GribDecoder&) {}
 };
 
-
 class NewLineHandler : public TitleFieldHandler {
 public:
     NewLineHandler() {}
@@ -2209,7 +2166,6 @@ public:
     void operator()(TitleField&, vector<string>& title, const GribDecoder&) { title.push_back(""); }
 };
 
-
 class SatelliteHandler : public TitleFieldHandler {
 public:
     SatelliteHandler() {}
@@ -2311,7 +2267,6 @@ public:
     }
 };
 
-
 class GribExpverHandler : public TitleFieldHandler {
 public:
     GribExpverHandler() {}
@@ -2327,7 +2282,6 @@ public:
     }
 };
 
-
 class GribEpsNumberInfoHandler : public TitleFieldHandler {
 public:
     GribEpsNumberInfoHandler() {}
@@ -2335,7 +2289,6 @@ public:
     void operator()(TitleField& field, vector<string>& title, const GribDecoder& grib) { title.back() += "epsnumber?"; }
 };
 
-
 class GribUnitHandler : public TitleFieldHandler {
 public:
     GribUnitHandler() {}
@@ -2377,7 +2330,6 @@ void GribDecoder::vComponent() {
 }
 }  // end namespace magics
 
-
 static SimpleObjectMaker<GribParamCriter, MatchCriteria> gribparamcriter("parameter");
 static SimpleObjectMaker<GribParamHandler, TitleFieldHandler> gribparamhandler("parameter");
 static SimpleObjectMaker<GribBaseDateHandler, TitleFieldHandler> gribbasedatehandler("base_date");
@@ -2403,12 +2355,10 @@ static SimpleObjectMaker<SatelliteHandler, TitleFieldHandler> satellitehandler("
 static SimpleObjectMaker<ChannelHandler, TitleFieldHandler> channelhandler("channel");
 static SimpleObjectMaker<GribBaseDateHandler, TitleFieldHandler> datehandler("date");
 
-
 static SimpleObjectMaker<GribObsDiagCriter, MatchCriteria> gribobsdiagriter("observationDiagnostic");
 static SimpleObjectMaker<GribObsDiagHandler, TitleFieldHandler> gribobsdiaghandler("observationDiagnostic");
 static SimpleObjectMaker<GribObstatHandler, TitleFieldHandler> gribobstathandler("obstat");
 
-
 static SimpleObjectMaker<GribLocalCriter, MatchCriteria> gribstreamcriter("stream");
 static SimpleObjectMaker<GribLocalCriter, MatchCriteria> gribtypecriter("type");
 static SimpleObjectMaker<GribLocalCriter, MatchCriteria> gribclasscriter("class");
@@ -2426,7 +2376,7 @@ static SimpleObjectMaker<GribLambertAzimutalInterpretor, GribInterpretor> lamber
     "lambert_azimuthal_equal_area");
 static SimpleObjectMaker<GribLambertInterpretor, GribInterpretor> lambert("lambert");
 static SimpleObjectMaker<GribPolarStereoInterpretor, GribInterpretor> polar("polar_stereographic");
-
+static SimpleObjectMaker<GribProjInterpretor, GribInterpretor> proj("proj");
 
 #include "GribSatelliteInterpretor.h"
 static SimpleObjectMaker<GribSatelliteInterpretor, GribInterpretor> satellite("space_view");
